from qtpy.QtGui import QPixmap as _QPixmap

from siriushlacon.vbc.consts import CHECK_IMG as _CHECK_IMG
from siriushlacon.vbc.consts import CNPEM_IMG as _CNPEM_IMG
from siriushlacon.vbc.consts import LNLS_IMG as _LNLS_IMG
from siriushlacon.vbc.consts import PLAY_IMG as _PLAY_IMG
from siriushlacon.vbc.consts import STOP_IMG as _STOP_IMG
from siriushlacon.vbc.consts import WARNING_IMG as _WARNING_IMG

CHECK = _QPixmap(_CHECK_IMG)
CNPEM = _QPixmap(_CNPEM_IMG)
LNLS = _QPixmap(_LNLS_IMG)
PLAY = _QPixmap(_PLAY_IMG)
STOP = _QPixmap(_STOP_IMG)
WARNING = _QPixmap(_WARNING_IMG)
