__version__ = "2.4.2"
__author__ = "Claudio F. Carneiro"
__date__ = "Fri, 27 Aug 2021 16:36:41 +0000"

DEFAULT_WINDOW_TITLE: str = "Sirius Interfaces"
GENERIC_LAUNCHER_FILE_NAME: str = "sirius-hla-as-ap-generic-launcher"
VBC_LAUNCHER_FILE_NAME: str = "sirius-hla-as-va-vbc"
MAIN_LAUNCHER_FILE_NAME: str = "sirius-hla-as-ap-conlauncher"
